@@ -28,34 +28,49 @@
 
   (define (with-mock-wrapper func)
     (lambda (obj)
-      (if (mock? obj)
-	  (dynamic-wind coverlets (lambda () (func (obj 'value))) openlets)
-	  (if (openlet? obj)           ; (with-let (mock-number 3) (abs (openlet (inlet 'abs (lambda (val) 0)))))
-	      (if (procedure? obj)     ; (display (openlet (with-let (mock-number 0) (lambda () 1))))
-		  (dynamic-wind coverlets (lambda () (func obj)) openlets)
-		  ;; TODO: and c-pointer? c-object?
-		  (let ((func-name (string->symbol (object->string func))))
-		    (if (procedure? (obj func-name))
-			((obj func-name) obj)
-			(func obj))))
-	      (func obj)))))
+      (cond ((mock? obj)
+	     (dynamic-wind
+		 coverlets
+		 (lambda ()
+		   (func (obj 'value)))
+		 openlets))
+
+	    ((not (openlet? obj))
+	     (func obj))
+
+	    ((procedure? obj) ; TODO: and c-pointer? c-object?
+	     (dynamic-wind
+		 coverlets
+		 (lambda ()
+		   (func obj))
+		 openlets))
+
+	    (else
+	     (let ((func-name (string->symbol (object->string func))))
+	       (if (procedure? (obj func-name))
+		   ((obj func-name) obj)
+		   (func obj)))))))
 
   (define (with-mock-wrapper* func)
     (lambda args
-      (let ((unknown-openlets #f))
-	(let ((new-args (map (lambda (arg)
-			       (if (mock? arg)
-				   (arg 'value)
-				   (begin
-				     (if (and (openlet? arg)
-					      (not (procedure? arg))
-					      (not (c-pointer? arg)))
-					 (set! unknown-openlets #t))
-				     arg)))
-			     args)))
-	  (if unknown-openlets
-	      (apply func new-args)
-	      (dynamic-wind coverlets (lambda () (apply func new-args)) openlets))))))
+      (let ((unknown-openlets #f)
+	    (new-args ()))
+	(for-each (lambda (arg) ; not map here because (values) should not be ignored: (+ (mock-number 4/3) (values))
+		    (set! new-args
+			  (cons (if (mock? arg)
+				    (arg 'value)
+				    (begin
+				      (if (and (openlet? arg)
+					       (not (procedure? arg))
+					       (not (macro? arg))
+					       (not (c-pointer? arg)))
+					  (set! unknown-openlets #t))
+				      arg))
+				new-args)))
+		  args)
+	(if unknown-openlets
+	    (apply func (reverse new-args))
+	    (dynamic-wind coverlets (lambda () (apply func (reverse new-args))) openlets)))))
 
   ;; one tricky thing here is that a mock object can be the let of with-let: (with-let (mock-port ...) ...)
   ;;   so a mock object's method can be called even when no argument is a mock object.  Even trickier, the
@@ -88,7 +103,12 @@
 		       
 		       'equivalent?        (with-mock-wrapper* #_equivalent?)
 		       'vector-ref         (with-mock-wrapper* #_vector-ref)
-		       'vector-length      (with-mock-wrapper #_length)
+		       'vector-length      (if (provided? 'pure-s7)
+					       (lambda (vect)
+						 (if (vector? vect)
+						     (length vect)
+						     (error 'wrong-type-arg "vector-length argument should be a vector: ~A" vect)))
+					       (with-mock-wrapper #_vector-length))
 		       'reverse            (with-mock-wrapper #_reverse)
 		       'sort!              (with-mock-wrapper* #_sort!)
 		       'make-iterator      (with-mock-wrapper #_make-iterator)
@@ -265,7 +285,13 @@
 						     (#_string-set! (obj 'value) i val)
 						     (error 'out-of-range "unknown field: ~S" i)))
 		       
-		       'string-length          (with-mock-wrapper #_length)
+		       'string-length          (if (provided? 'pure-s7)
+						   (lambda (str)
+						     (if (string? str)
+							 (length str)
+							 (error 'wrong-type-arg "string-length argument should be a string: ~A" str)))
+						   (with-mock-wrapper #_string-length))
+
 		       'string-append          (with-mock-wrapper* #_string-append)
 		       'string-copy            (with-mock-wrapper #_copy)
 		       
@@ -751,19 +777,23 @@
 		       'let-ref-fallback (lambda (obj ind) 
 					   (if (eq? ind 'value)
 					       #<undefined>
-					       (let ((val (begin 
-							    (coverlet obj)
-							    (#_list-ref (obj 'value) ind))))
-						 (openlet obj) 
-						 val)))
+					       (if (integer? ind)
+						   (let ((val (begin 
+								(coverlet obj)
+								(#_list-ref (obj 'value) ind))))
+						     (openlet obj) 
+						     val)
+						   (error "let-ref mock-pair index is not an integer: ~S" ind))))
 		       'let-set-fallback (lambda (obj ind val) 
 					   (if (eq? ind 'value)
 					       #<undefined>
-					       (let ((val (begin 
-							    (coverlet obj)
-							    (#_list-set! (obj 'value) ind val))))
-						 (openlet obj)
-						 val)))
+					       (if (integer? ind)
+						   (let ((val (begin 
+								(coverlet obj)
+								(#_list-set! (obj 'value) ind val))))
+						     (openlet obj)
+						     val)
+						   (error "let-set! mock-pair index is not an integer: ~S" ind))))
 		       
 		       'reverse!         (lambda (obj) 
 					   (if (mock-pair? obj)
@@ -860,8 +890,7 @@
 		       )))
 	  
 	  (define (mock-symbol s)
-	    (if (and (symbol? s)
-		     (not (let? s)))
+	    (if (symbol? s)
 		(immutable!
 		 (openlet
 		  (sublet (*mock-symbol* 'mock-symbol-class)
